#!/usr/bin/env python3

import sys
from akari.gui import init_gui
from akari.__init__ import __version__
from akari.utils import handle_flags, loadDB, scan_diretory

def main():
    dirname = handle_flags()
    if dirname == -2:
        print("akari version {}".format(__version__))
        sys.exit(0)
    if dirname == -1:
        init_gui()
    else:
        db = loadDB()
        scan_diretory(dirname, db)
        sys.exit(0)

if __name__ == '__main__':
    main()



import os
import sys
import json
import argparse
import requests
from pathlib import Path
from shutil import copyfile
from bs4 import BeautifulSoup

image_extensions = ['.jpg','.jpeg','.png','.gif']
data_dir = str(Path.home()) + '/.config/akari'

def query_iqdb(filename):
    url = 'https://iqdb.org'
    files = {'file': (filename, open(filename, 'rb'))}
    res = requests.post(url,files=files)
    return res

def update_tags(tags, db):
    for tag in tags:
        if tag in db['akari-tags']:
            db['akari-tags'][tag] = db['akari-tags'][tag] + 1
        else:
            db['akari-tags'][tag] = 1

def commit_changes(db):
    with open(data_dir+'/db.json', 'w') as outfile:
        json.dump(db, outfile)

def parse_result(result):
    soup = BeautifulSoup(result.text, 'html.parser')
    try:
        tables = soup.find_all('table')
        search_result = tables[1].findChildren("th")[0].get_text()
    except:
        print('Unexpected Error')
        return ['server-error']
    tags = []
    if search_result == 'No relevant matches':
        print('No Match Found')
        tags.append('undefined')
    else:
        print('Match Found')
        alt_string = tables[1].findChildren("img")[0]['alt']
        tag_string_index = alt_string.find('Tags:')
        if tag_string_index == -1:
            tags.append('undefined')
        else:
            tag_string = alt_string[tag_string_index+6:]
            tag_string = tag_string.lower()
            tag_string_formatted  = ''.join(c for c in tag_string if c not in ',')
            tags = list(set(tag_string_formatted.split(" ")))
    return tags

def scan_diretory(dirname, db):
    image_paths = []
    count = 1
    for root, d_names, f_names in os.walk(dirname):
        for f_name in f_names:
            if(f_name[-4:] in image_extensions):
                fullpath = os.path.join(root, f_name)
                image_paths.append(fullpath)
    num_of_images = len(image_paths)
    for image in image_paths:
        filename = os.path.basename(os.path.normpath(image))
        print("Processing Image {} of {}: {}".format(count, num_of_images, filename))
        count = count + 1
        if image in db and db[image] != ['server-error']:
            print('Already exits in DB. Skipping...')
        else:
            result = query_iqdb(image)
            tags = parse_result(result)
            db[image] = tags
            update_tags(tags, db)
            commit_changes(db)
            print(tags)
        print("-----x--x-----")

def loadDB():
    db = {}
    db['akari-tags'] = {}
    misplaced_files = []
    tags_to_remove = []
    db_location = data_dir + '/db.json'

    if not os.path.exists(data_dir):
        os.makedirs(data_dir)
    if os.path.isfile(db_location):
        copyfile(db_location,db_location+'.backup')
        with open(db_location) as json_db:
            db = json.load(json_db)

    for key in db:
        if key == 'akari-tags':
            continue
        if not os.path.isfile(key):
            misplaced_files.append(key)
    for file in misplaced_files:
        for tag in db[file]:
            db['akari-tags'][tag] = db['akari-tags'][tag] - 1
            if db['akari-tags'][tag] == 0:
                tags_to_remove.append(tag)
        db.pop(file, None)
        print('{} misplaced'.format(file))
    for tag in tags_to_remove:
        db['akari-tags'].pop(tag, None)
        print('{} tag removed'.format(tag))

    commit_changes(db)

    return db

def handle_flags():
    parser = argparse.ArgumentParser()
    parser.add_argument('-s','--scan',metavar='/path/to/dir',help='Scan directory for new images',default=None)
    parser.add_argument('-g','--gui',help='Start the GUI',action='store_true')
    parser.add_argument('-v','--version',help='Displays the version',action='store_true')
    args = parser.parse_args()
    dirname = args.scan

    if args.version == True:
        return -2
    if args.gui == True:
        return -1
    if dirname is None:
        parser.print_help(sys.stderr)
        sys.exit(2)
    if not os.path.isdir(dirname):
        print("Invalid Path")
        parser.print_help(sys.stderr)
        sys.exit(2)

    return dirname



import sys
from akari.utils import loadDB, commit_changes
from PyQt5.QtCore import QSize
from PyQt5.QtGui import QIcon, QColor, QFont, QPixmap
from PyQt5.QtWidgets import QApplication, QWidget, QPushButton, QListWidget, QListWidgetItem, QListView, QHBoxLayout, QVBoxLayout, QAbstractItemView, QMenu, QInputDialog, QLineEdit

class Main(QWidget):
    def __init__(self):
        super().__init__()
        self.setWindowTitle('akari')
        self.setWindowIcon(QIcon('static/icon.jpg'))
        self.db = None

        self.mainLayout = QHBoxLayout()

        self.sidePanel = QVBoxLayout()
        self.tagList = QListWidget()
        self.filerButton = QPushButton("Filter")
        self.clearFilerButton = QPushButton("Reset")
        self.sidePanel.addWidget(self.tagList)

        self.imagePanel = QVBoxLayout()
        self.imageList = QListWidget()
        self.imagePanelOptions = QHBoxLayout()
        self.miscButton1 = QPushButton("Misc Option 1")
        self.miscButton2 = QPushButton("Misc Option 2")
        self.imagePanel.addWidget(self.imageList)

        self.mainLayout.addLayout(self.sidePanel)
        self.mainLayout.addLayout(self.imagePanel)
        self.setLayout(self.mainLayout)

        self.initUI()

    def initUI(self):
        self.sidePanel.addWidget(self.filerButton, 1)
        self.sidePanel.addWidget(self.clearFilerButton, 1)
        self.tagList.setSelectionMode(QAbstractItemView.ExtendedSelection)

        self.imageList.setViewMode(QListView.IconMode)
        self.imageList.setIconSize(QSize(300,300))
        self.imageList.installEventFilter(self)

        self.imagePanel.addLayout(self.imagePanelOptions)
        self.imagePanelOptions.addWidget(self.miscButton1)
        self.imagePanelOptions.addWidget(self.miscButton2)

        self.addEventHandlers()
        self.show()

    def updateDB(self, db):
        self.db = db

    def addEventHandlers(self):
        self.filerButton.clicked.connect(self.filterButton_pressed)
        self.clearFilerButton.clicked.connect(self.clearFilterButton_pressed)

    def addTagEditingButtons(self):
        self.addTagsButton = QPushButton("Add New Tag")
        self.removeTagsButton = QPushButton("Remove Tags")
        self.sidePanel.addWidget(self.addTagsButton)
        self.sidePanel.addWidget(self.removeTagsButton)
        self.addTagsButton.clicked.connect(self.addTagsButton_pressed)
        self.removeTagsButton.clicked.connect(self.removeTagsButton_pressed)

    def removeTagEditingButtons(self):
        try:
            self.sidePanel.removeWidget(self.addTagsButton)
            self.addTagsButton.deleteLater()
            self.sidePanel.removeWidget(self.removeTagsButton)
            self.removeTagsButton.deleteLater()
        except:
            pass

    def refreshTagListForImage(self, image):
        self.tagList.clear()
        for tag in self.db[image]:
            self.tagList.addItem(tag)

    def addTagsButton_pressed(self):
        image = self.imageList.item(0).data(0)
        tag, okPressed = QInputDialog.getText(self, "Enter Tag","Enter Tag", QLineEdit.Normal, "")
        if okPressed and tag != '':
            if tag not in self.db[image]:
                self.db[image].append(tag)
                if tag in self.db['akari-tags']:
                    self.db['akari-tags'][tag] = self.db['akari-tags'][tag] + 1
                else:
                    self.db['akari-tags'][tag] = 1
        commit_changes(self.db)
        self.db = loadDB()
        self.refreshTagListForImage(image)

    def removeTagsButton_pressed(self):
        tags_to_remove = self.tagList.selectedItems()
        image = self.imageList.item(0).data(0)
        for tag in tags_to_remove:
            tag = tag.text()
            self.db[image].remove(tag)
            self.db['akari-tags'][tag] = self.db['akari-tags'][tag] - 1
            if self.db['akari-tags'][tag] == 0:
                self.db['akari-tags'].pop(tag,None)
        commit_changes(self.db)
        self.db = loadDB()
        self.refreshTagListForImage(image)

    def filterButton_pressed(self):
        selected_items = self.tagList.selectedItems()
        selected_tags = []
        filtered_images = []
        for i in range(len(selected_items)):
            selected_tags.append(str(self.tagList.selectedItems()[i].text()).split(" ")[-1])
        for imgPath in self.db:
            if imgPath == 'akari-tags':
                continue
            if all(elem in self.db[imgPath] for elem in selected_tags):
                if imgPath not in filtered_images:
                    filtered_images.append(imgPath)
        self.addImages(filtered_images)
        self.removeTagEditingButtons()

    def clearFilterButton_pressed(self):
        imgList = self.getImgList()
        self.addImages(imgList)
        self.addTags()
        self.removeTagEditingButtons()

    def viewMetadata(self):
        selected_image = self.imageList.selectedItems()[0].data(0)
        tags = self.db[selected_image]
        self.tagList.clear()
        self.tagList.addItems(tags)
        self.addImages([selected_image])
        self.addTagEditingButtons()

    def contextMenuEvent(self, event):
        menu = QMenu(self)
        if self.imageList.selectedItems() != []:
            viewMetadata = menu.addAction("View Metadata")
            action = menu.exec_(self.mapToGlobal(event.pos()))
            if action == viewMetadata:
                self.viewMetadata()

    def getImgList(self):
        imgList = []
        for imgPath in self.db:
            if imgPath == 'akari-tags':
                continue
            imgList.append(imgPath)
        return imgList

    def addImages(self, imgList):
        self.imageList.clear()
        for imgPath in imgList:
            item = QListWidgetItem()
            icon = QIcon()
            pixmap = QPixmap(imgPath)
            icon.addPixmap(pixmap)
            item.setFont(QFont("Times",1))
            item.setForeground(QColor("white"))
            item.setText(imgPath)
            item.setIcon(icon)
            self.imageList.addItem(item)

    def addTags(self):
        self.tagList.clear()
        tagList = []
        for tag in sorted(self.db['akari-tags'], key=self.db['akari-tags'].get, reverse=True):
            tagList.append(str(self.db['akari-tags'][tag]) + ' ' + tag)
        self.tagList.addItems(tagList)

def init_gui():
    db = loadDB()
    app = QApplication(sys.argv)
    main = Main()
    main.updateDB(db)
    main.addTags()
    imgList = main.getImgList()
    main.addImages(imgList)
    sys.exit(app.exec_())



import setuptools
from akari import __version__, __author__

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="akari",
    version=__version__,
    author=__author__,
    author_email="mananapr@gmail.com",
    description="Program to manage and tag your anime artwork",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/mananapr/akari",
    packages=setuptools.find_packages(),
    install_requires=[
        'requests',
        'pyqt5',
        'beautifulsoup4',
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
    ],
    entry_points={
        'console_scripts': [
            'akari = akari.cli:main'
        ]
    }
)
